import json
from Microsoft.Finance.Common.Spark import Spark

class TestObject(object):

    #region Constructor
    
    def __init__(self, name: str, database: str, correlationId: int):
        self.__name = name
        self.__database = database
        self.__correlationId = correlationId
        self.__rowCountBefore = -1
        self.__rowCountAfter = -1
    
    #endregion

    #region Properties

    @property
    def Database(self):
        return self.__database

    @property
    def CorrelationId(self):
        return self.__correlationId

    @property
    def Name(self):
        return self.__name

    @property
    def Type(self):
        if self.__name[0:4] == "DIM_":
            return "dimension"
        elif self.__name[0:5] == "FACT_":
            return "fact"
        else:
            return "unspecified - staging"

    #endregion

    #region Class Methods

    def __str__(self):

        d = {}

        d["name"] = self.__name
        d["database"] = self.__database
        d["type"] = self.Type
        d["row_count_before"] = self.__rowCountBefore
        d["row_count_after"] = self.__rowCountAfter

        return json.dumps(d)

    def set_row_count_before(self, spark: Spark = None, rowCount = -1):

        if rowCount >= 0:
            self.__rowCountBefore = rowCount
        else:
            self.get_row_count(spark)

    def set_row_count_after(self, spark: Spark = None, rowCount = -1):

        if rowCount >= 0:
            self.__rowCountBefore = rowCount
        else:
            self.get_row_count(spark, False)

    def get_row_count(self, spark: Spark, before: bool = True):

        if not spark:
            raise ValueError("spark utility cannot be None...")

        rowCount = spark.count_table_or_view(self.__database, self.__database)

        if before:
            self.__rowCountBefore = rowCount
        else:
            self.__rowCountAfter = rowCount

    #endregion


from Microsoft.Finance.DataStandards.UnitTest.TestCase.CommonTestCase import CommonTestCase
from Microsoft.Finance.DataStandards.UnitTest.TestCase.FactTestCase import FactTestCase
from Microsoft.Finance.DataStandards.UnitTest.TestSuite.BaseTestSuite import BaseTestSuite
from pyspark.sql.session import SparkSession

class FactTestSuite(BaseTestSuite):

    def __init__(self, sparkSession: SparkSession, databaseName: str, tableName: str, linkedDimensions: [(str, str)], excludeFromAggregation: [str] = []):
        super(FactTestSuite, self).__init__(sparkSession, databaseName, tableName)
        self.__linkedDimensions = linkedDimensions
        self.__excludeFromAggregations = excludeFromAggregations

    def queue_tests(self) -> None:
        factTestCase = FactTestCase(self.SparkSession, self.TableDataFrame, self.FieldInfo, self.__linkedDimensions, self.DatabaseName, self.TableName, self.TableCount, self.CustomEvents, self.__excludeFromAggregations)
        commonTestCase = CommonTestCase(self.TableDataFrame, self.FieldInfo, self.CustomEvents)
        self.TestSuite.addTest(factTestCase)
        self.TestSuite.addTest(commonTestCase)

    def run_tests(self) -> None:
        self.TestRunner.run(self.TestSuite)



from Microsoft.Finance.Common.Utilities import Utilities
from Microsoft.Finance.Common.Spark import Spark
from Microsoft.Finance.DataStandards.Utilities.TestEvent import TestEvent
from Microsoft.Finance.DataStandards.Utilities.TestUtilities import TestUtilities

import unittest

class DimensionTestCase(unittest.TestCase):

    #region Constructor

    def __init__(self, sparkUtility: Spark, fieldInfo: {(str, type)}, databaseName: str, tableName: str, tableCount: int, customEvents: [TestEvent], methodName: str = "runTest") -> None:
        super(DimensionTestCase, self).__init__(methodName)
        self.__spark = sparkUtility
        self.__databaseName = databaseName
        self.__tableName = tableName
        self.__tableCount = tableCount
        self.__customEvents = customEvents
        self.__fieldInfo = fieldInfo

    #endregion

    #region Class Methods

    def test_primary_key_unique_values(self) -> None:

        '''
        desc
            Conventions for dimension tables
                g. Always check to make sure that there are no duplicates in Id column of dimension
        '''

        event = TestEvent(Utilities.get_function_caller_name())

        eventCode1 = "key-with-unique-values"
        eventCode2 = "key-without-unique-values"
        eventCode3 = "non-key-columns"

        eventResults1 = []
        eventResults2 = []
        eventResults3 = []

        event.start()

        for columnName, columnType in self.__fieldInfo:
            if TestUtilities.check_dimension_key_name_and_type(columnName, columnType):
                keyValueCount = self.__spark.count_distinct_column_values(self.__databaseName, self.__tableName, columnName)
                if keyValueCount == tableCount:
                    eventResults1.append((columnName, columnType))
                else:
                    eventResults2.append((columnName, columnType))
            else:
                eventResults3.append((columnName, columnType))
        
        event.end()

        event.add_note(eventCode1, eventResults1)
        event.add_note(eventCode2, eventResults2)
        event.add_note(eventCode3, eventResults3)

        self.__customEvents.append(event)

        with self.subTest():
            self.assertEqual(len(eventResults1), 1)
        
        with self.subTest():
            self.assertEqual(len(eventResults2), 0)
    
    def test_primary_key(self) -> None:
        
        '''
        desc
            Conventions for dimension tables
                c. Unique column of table should be named as DIM_DimensionNameId and should always be of INT or BIGINT datatype
                d. Only the unique column of dimension should have “DIM” in name
        '''
        
        event = TestEvent(Utilities.get_function_caller_name())

        eventCode1 = "key-column"
        eventCode2 = "key-bad-type"
        eventCode3 = "non-key-columns"

        eventResults1 = []
        eventResults2 = []
        eventResults3 = []

        event.start()

        for columnName, columnType in self.__fieldInfo:
            if TestUtilities.check_dimension_key_name(columnName):
                if TestUtilities.check_dimension_key_type(columnType):
                    eventResults1.append((columnName, columnType))
                else:
                    eventResults2.append((columnName, columnType))
            else:
                eventResults3.append((columnName, columnType))
        
        event.end()
        
        event.add_note(eventCode1, eventResults1)
        event.add_note(eventCode2, eventResults2)
        event.add_note(eventCode3, eventResults3)

        self.__customEvents.append(event)

        with self.subTest():
            self.assertEqual(len(eventResults1), 1)
        
        with self.subTest():
            self.assertEqual(len(eventResults2), 0)
        
        with self.subTest():
            self.assertGreater(len(eventResults3), 0)

    #endregion


from datetime import datetime, timedelta
from json import dumps
from typing import Any
from uuid import uuid1

class TestEvent(object):

    #region Constructor

    def __init__(self, testName: str):
        self.__testName = testName
        self.__notes = {}
        self.__start = None
        self.__end = None
        self.__duration = None

    #endregion

    #region Overloads
    
    def __str__(self) -> str:
        s = dumps(self.__dict__)
        return s
    
    #endregion
    
    #region Properties

    @property
    def TestName(self) -> str:
        return self.__testName

    @property
    def Notes(self) -> {}:
        return self.__notes

    @property
    def Start(self) -> datetime:
        return self.__start

    @property
    def End(self) -> datetime:
        return self.__end

    @property
    def EventDurationSeconds(self) -> float:
        if not self._start or self._end:
            return 0.0
        else:
            if not self.__duration:
                self.__duration = self.__end - self.__start
            return self.__duration.total_seconds()

    #endregion

    #region Class Methods

    def start(self) -> None:
        self.__start = datetime.now()
   
    def end(self) -> None:
        self.__end = datetime.now()

    def add_note(self, key: str, val: Any) -> None:

        if not key:
            raise ValueError("key cannot be None or empty...")

        if key in self.__notes.keys():
            key = "{}-{}".format(key, uuid1())

        self.__notes[key] = val

    #endregion